/* Created by Nitesh Kumar Verma on 15/12/23 */

import 'package:exide_crr/res/app_color_text_size.dart';
import 'package:flutter/material.dart';
import 'package:flutter/services.dart';

typedef OnCodeEnteredCompletion = void Function(String value);
typedef OnCodeChanged = void Function(String value);
typedef HandleControllers = void Function(List<TextEditingController?> controllers);

// ignore: must_be_immutable
class OtpTextField extends StatefulWidget {
  /// show cursor of pin text field
  final bool showCursor;

  /// number of pin input for otp
  final int numberOfFields;

  /// pin input width
  final double fieldWidth;

  /// pin input height
  final double fieldHeight;

  /// border width of pin input
  final double borderWidth;

  /// color code for enabled border of pin input
  final Color enabledBorderColor;

  /// color code for focused border of pin input
  final Color focusedBorderColor;

  /// color code for disabled border of pin input
  final Color disabledBorderColor;

  /// color code for border of pin input
  final Color borderColor;

  /// color code for cursor of pin input
  final Color? cursorColor;

  /// margin of pin input
  final EdgeInsetsGeometry margin;

  /// pin input type
  /// [TextInputType.text], [TextInputType.number]
  final TextInputType keyboardType;

  /// text style for pin input
  final TextStyle? textStyle;

  /// Main axis alignment [MainAxisAlignment.start], [MainAxisAlignment.center]
  /// [MainAxisAlignment.spaceBetween], [MainAxisAlignment.spaceEvenly] etc
  final MainAxisAlignment mainAxisAlignment;

  /// cross axis alignment [CrossAxisAlignment.start], [CrossAxisAlignment.center]
  final CrossAxisAlignment crossAxisAlignment;

  /// The callback called when code submitted
  final OnCodeEnteredCompletion? onSubmit;

  /// The callback called when code change
  final OnCodeChanged? onCodeChanged;

  /// Controllers to handle otp text fields
  final HandleControllers? handleControllers;

  /// Obscure text, password like input or not
  final bool obscureText;

  /// Show input pin as box or not
  final bool showFieldAsBox;

  /// pin input field enabled or not
  final bool enabled;

  /// pin input field filled or not
  final bool filled;

  /// pin input field autofocus or not
  final bool autoFocus;

  /// pin input field readOnly or not
  final bool readOnly;

  /// pin input field clearText enabled or not
  bool clearText;

  /// pin input field has custom input decoration enabled or not
  final bool hasCustomInputDecoration;

  /// pin input field fill Color
  final Color fillColor;

  /// pin input field border radius
  final BorderRadius borderRadius;

  /// pin input field input decoration
  final InputDecoration? decoration;

  /// List of styles for every input of OTP
  final List<TextStyle?> styles;

  /// List of TextInputFormatter for every input of OTP
  final List<TextInputFormatter>? inputFormatters;

  OtpTextField({super.key,
    this.showCursor = true,
    this.numberOfFields = 4,
    this.fieldWidth = 40.0,
    this.fieldHeight = 40.0,
    this.margin = const EdgeInsets.only(right: 8.0),
    this.textStyle,
    this.clearText = false,
    this.styles = const [],
    this.keyboardType = TextInputType.number,
    this.borderWidth = 2.0,
    this.cursorColor,
    this.disabledBorderColor = const Color(0xFFE7E7E7),
    this.enabledBorderColor = const Color(0xFFE7E7E7),
    this.borderColor = const Color(0xFFE7E7E7),
    this.focusedBorderColor = const Color(0xFF4F44FF),
    this.mainAxisAlignment = MainAxisAlignment.center,
    this.crossAxisAlignment = CrossAxisAlignment.center,
    this.handleControllers,
    this.onSubmit,
    this.obscureText = false,
    this.showFieldAsBox = false,
    this.enabled = true,
    this.autoFocus = false,
    this.hasCustomInputDecoration = false,
    this.filled = false,
    this.fillColor = const Color(0xFFFFFFFF),
    this.readOnly = false,
    this.decoration,
    this.onCodeChanged,
    this.borderRadius = const BorderRadius.all(Radius.circular(4.0)),
    this.inputFormatters,
  })  : assert(numberOfFields > 0),
        assert(styles.isNotEmpty
            ? styles.length == numberOfFields
            : styles.isEmpty);

  @override
  OtpTextFieldState createState() => OtpTextFieldState();
}

class OtpTextFieldState extends State<OtpTextField> {
  /// List of verification code
  /// [_verificationCode.length] = [numberOfFields]
  late List<String?> _verificationCode;

  /// List of focus node for otp text field
  /// [_focusNodes.length] = [numberOfFields]
  late List<FocusNode?> _focusNodes;

  /// List of text edit controllers for otp text field
  /// [_textControllers.length] = [numberOfFields]
  late List<TextEditingController?> _textControllers;

  @override
  void initState() {
    super.initState();

    /// initialization
    _verificationCode = List<String?>.filled(widget.numberOfFields, null);
    _focusNodes = List<FocusNode?>.filled(widget.numberOfFields, null);
    _textControllers = List<TextEditingController?>.filled(
      widget.numberOfFields,
      null,
    );
  }

  @override
  void didUpdateWidget(covariant OtpTextField oldWidget) {
    super.didUpdateWidget(oldWidget);
    if(oldWidget.clearText != widget.clearText && widget.clearText == true) {
      for (var controller in _textControllers ){
        controller?.clear();
      }
      _verificationCode = List<String?>.filled(widget.numberOfFields, null);
      setState((){
        widget.clearText = false;
      });
    }
  }

  @override
  void dispose() {
    super.dispose();
    /// dispose the all controllers
    for (var controller in _textControllers) {
      controller?.dispose();
    }
  }

  @override
  Widget build(BuildContext context) {
    /// generate the text fields length of [numberOfFields]
    return generateTextFields(context);
  }

  Widget _buildTextField({
    required BuildContext context,
    required int index,
    TextStyle? style,
  }) {
    return Expanded(
      child: Container(
        width: widget.fieldWidth,
        margin: widget.margin,
        height: widget.fieldHeight,
        child: TextField(
          showCursor: widget.showCursor,
          keyboardType: widget.keyboardType,
          textAlign: TextAlign.center,
          maxLength: 1,
          cursorWidth: 2,
          readOnly: widget.readOnly,
          style: style ?? widget.textStyle,
          autofocus: widget.autoFocus,
          cursorColor: widget.cursorColor,
          controller: _textControllers[index],
          focusNode: _focusNodes[index],
          enabled: widget.enabled,
          inputFormatters: widget.inputFormatters,
          decoration: widget.hasCustomInputDecoration
              ? widget.decoration
              : InputDecoration(
            counterText: "",
            focusColor: AppColor.whiteColor,
            filled: widget.filled,
            fillColor: AppColor.whiteColor,
            contentPadding: const EdgeInsets.only(bottom: 12),
            focusedBorder: widget.showFieldAsBox
                ? outlineBorder(widget.focusedBorderColor)
                : underlineInputBorder(widget.focusedBorderColor),
            enabledBorder: widget.showFieldAsBox
                ? outlineBorder(widget.enabledBorderColor)
                : underlineInputBorder(widget.enabledBorderColor),
            disabledBorder: widget.showFieldAsBox
                ? outlineBorder(widget.disabledBorderColor)
                : underlineInputBorder(widget.disabledBorderColor),
            border: widget.showFieldAsBox
                ? outlineBorder(widget.borderColor)
                : underlineInputBorder(widget.borderColor),
          ),
          obscureText: widget.obscureText,
          onChanged: (String value) {
            //save entered value in a list
            _verificationCode[index] = value;
            onCodeChanged(verificationCode: value);
            changeFocusToNextNodeWhenValueIsEntered(
              value: value,
              indexOfTextField: index,
            );
            changeFocusToPreviousNodeWhenValueIsRemoved(value: value, indexOfTextField: index);
            onSubmit(verificationCode: _verificationCode);
          },
        ),
      ),
    );
  }

  OutlineInputBorder outlineBorder(Color color) {
    return OutlineInputBorder(
      borderSide: BorderSide(
        width: widget.borderWidth,
        color: color,
      ),
      borderRadius: widget.borderRadius,
    );
  }

  UnderlineInputBorder underlineInputBorder(Color color) {
    return UnderlineInputBorder(
      borderSide: BorderSide(
        color: color,
        width: widget.borderWidth,
      ),
    );
  }

  Widget generateTextFields(BuildContext context) {
    List<Widget> textFields = List.generate(widget.numberOfFields, (int i) {
      addFocusNodeToEachTextField(index: i);
      addTextEditingControllerToEachTextField(index: i);

      if (widget.styles.isNotEmpty) {
        return _buildTextField(
          context: context,
          index: i,
          style: widget.styles[i],
        );
      }
      if (widget.handleControllers != null) {
        widget.handleControllers!(_textControllers);
      }
      return _buildTextField(context: context, index: i);
    });


    return Row(
      mainAxisAlignment: widget.mainAxisAlignment,
      crossAxisAlignment: widget.crossAxisAlignment,
      children: textFields,
    );
  }

  void addFocusNodeToEachTextField({required int index}) {
    if (_focusNodes[index] == null) {
      _focusNodes[index] = FocusNode();
    }
  }

  void addTextEditingControllerToEachTextField({required int index}) {
    if (_textControllers[index] == null) {
      _textControllers[index] = TextEditingController();
    }
  }

  void changeFocusToNextNodeWhenValueIsEntered({
    required String value,
    required int indexOfTextField,
  }) {
    //only change focus to the next textField if the value entered has a length greater than one
    if (value.isNotEmpty) {
      //if the textField in focus is not the last textField,
      // change focus to the next textField
      if (indexOfTextField + 1 != widget.numberOfFields) {
        //change focus to the next textField
        FocusScope.of(context).requestFocus(_focusNodes[indexOfTextField + 1]);
      } else {
        //if the textField in focus is the last textField, unFocus after text changed
        _focusNodes[indexOfTextField]?.unfocus();
      }
    }
  }

  void changeFocusToPreviousNodeWhenValueIsRemoved({
    required String value,
    required int indexOfTextField,
  }) {
    //only change focus to the previous textField if the value entered has a length zero
    if (value.isEmpty) {
      //if the textField in focus is not the first textField,
      // change focus to the previous textField
      if (indexOfTextField != 0) {
        //change focus to the next textField
        FocusScope.of(context).requestFocus(_focusNodes[indexOfTextField - 1]);
      }
    }
  }


  void onSubmit({required List<String?> verificationCode}) {
    if (verificationCode.every((String? code) => code != null && code != '')) {
      if (widget.onSubmit != null) {
        widget.onSubmit!(verificationCode.join());
      }
    }
  }

  void onCodeChanged({required String verificationCode}) {
    if (widget.onCodeChanged != null) {
      widget.onCodeChanged!(verificationCode);
    }
  }
}